/* Version of Feb 15, 2002.  
Writes cif, with inversions
Numbers atoms from #1.  Includes 2 algor choices 
Prefers algor=2, corrected for Silvina's factor of 2

compile: gcc pssp.c psspdif.c putatoms.c -lm -o pssp */

#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "pssp.h"

float lambda, cellconv[3][3], cella, cellb, cellc, alpha, beta, gama,sumiobs;
float volume=0;
int nsym, centrosym, nintens, nats, npar;
char  title[100], space_group_symbol[10], putcmd[MAXPUTCMD][MAXBUF];
struct molecule molecule[NMOL];
struct atom atoms[NATS];
struct desc_range d_range;
struct symelement symelement[NSYM];
struct symxyz symtext[NSYM];
struct symxyz syminv[NSYM];
struct intens intens[NINTENS];
float ac[MAXATNUM][9];

/*--------------------------------------------------------------------------------------------------------------------*/
/* MAIN FUNCTION */
/*--------------------------------------------------------------------------------------------------------------------*/

main(){

  struct descr d0, dtest, dbest;
  int  ncyc,  cyc, nacc, ok, natinmol, h, i, j, nmol, idum;
  float sinth, t, r, dif, boltz, scale, sumr, sumrsq, s;
  int jumppar=1, algor=2;   /* parameter characterizing preferance for small jumps (jumppar) and algorithm number (algor)*/
  float init=0, fint=0, tfac=0;   /* initial and final temperature, factor to multiply it between cycles */
  float t1, t2, span;
  struct symelement *sp;
  struct symelement record;
  float p0min, p0max, p0step, p1min, p1max, p1step, gp0, gp1;
  int gp0n, gp1n;
  struct pt_rec rot1 = {0,0,0};
  struct pt_rec rot2 = {0,0,0};
  struct pt_rec rot3 = {0,0,0};
  struct atom *ap;
  int atype[MAXATNUM], atnum[NATOMTYPE], nextatomtype;
  char bufr[MAXBUF], *cp, *tp;
  char word1[20],word2[20],word3[20];
  char invword1[20],invword2[20],invword3[20];
  FILE *inf, *lebail, *logf, *ful, *gda, *ortep3, *cif;
  time_t time1, time2;
  
  for(i=0; i<MAXATNUM; i++) atype[i] = -1; // initialize some stuff for atom table
  for(i=0; i<NATOMTYPE; i++) atnum[i] = 0;
  nextatomtype = 0;
  
  costabstuff();
  
  time(&time1);
  srand(time1);
  printf("Started %s\n", ctime(&time1));
  
  printf("============================================================================================\n");
  printf("                                POWDER STRUCTURE SOLUTION PROGRAM                           \n");
  printf("============================================================================================\n");
  printf("Version Feb 15, 2002, by PWS, SP, AH\n");  
  printf("\n\n");
  
  
  /* ------------------------------------------ IT READS COMMANDS FROM THE INPUT FILE */
  
  while ( fgets(bufr, MAXBUF, stdin) != NULL) {    
    cp = bufr;
    while(*cp != '\n' && *cp != '\0') cp++;
    *cp = '\0';
    fflush(stdout);
    
    /* it reads the experiment title */
    if( strncmp (bufr, "title", 5) == 0 ){                          
      strncpy(title, bufr+6, 100);

      printf("EXPERIMENT: %s \n", title);
      printf("---------- \n");
      printf("\n");
      continue;
    }

    //Reads integrated intensities from FULLPROF or GSAS files 

    /* get Le Bail intensities from a FULLPROF .hkl file */
    if( strncmp (bufr, "lebailf", 7) == 0){
      init_pluck(bufr+8);
      
      if( (lebail = fopen( pluck(), "r") ) == NULL)
	errout("Can't open FULLPROF Le Bail file");
      if( (nintens = atoi( pluck() ) ) <= 0) errout("Can't understand nintens");
      if( nintens > NINTENS ) errout("RECOMPILE -- nintens > NINTENS");

      /* READ THE INTEGRATED INTENSITIES FROM THE LE BAIL FIT (FULLPROF) */

      printf("DATA READ FROM THE LE BAIL FIT: %s\n", bufr+7);
      printf("------------------------------ \n");      
      printf("  h   k   l     mult.   sigma   2theta   Iobs.  Overlap coefficients (Fi,i Fi,i+1 Fi,i+2 Fi,i+3 Fi,i+4)\n"); 
      printf("-------------------------------------------------------------------------------------------------------\n");
      
      fgets(bufr, MAXBUF, lebail);                                    /* skip the first two lines */
      fgets(bufr, MAXBUF, lebail);
      
      for(i=0; i<nintens; i++){
	if (fgets(bufr, 200, lebail) == NULL) break;
	sscanf(bufr, "%*d %d %d %d %d %*f %f %f %f", &intens[i].i_hkl[0], &intens[i].i_hkl[1], &intens[i].i_hkl[2], &intens[i].i_m, &intens[i].i_tth, &intens[i].i_wid, &intens[i].i_iobs);

	intens[i].i_wid /= 2.35; /* read in FWHM, convert to is sigma */

	if(i < NINTENS - NOVLP) for(j=1; j<NOVLP; j++) intens[i].i_cij[j] = 0;
	intens[i].i_cij[0] = 1. / (3.5449077 * intens[i].i_wid);
	for(j=1; j<NOVLP; j++){
	  if(i-j < 0) continue;
	  t1 = intens[i-j].i_wid * intens[i-j].i_wid + intens[i].i_wid * intens[i].i_wid;
	  t2 = intens[i-j].i_tth - intens[i].i_tth;
	  t2 *= t2 / (2. * t1);
	  if ( t2 > 10 ) intens[i-j].i_cij[j] = 0;
	  else intens[i-j].i_cij[j] = 1. / (2.506628 * sqrt(t1) * exp(t2) ); 
	}    
      }
      for(i=0; i<nintens; i++){
	printf("%3d %3d %3d %8d\t %3.3f\t %3.3f\t %5.1f ", intens[i].i_hkl[0], intens[i].i_hkl[1], intens[i].i_hkl[2], intens[i].i_m, intens[i].i_wid, intens[i].i_tth, intens[i].i_iobs);
	for(j=0; j<NOVLP; j++) printf("%8.4f ", intens[i].i_cij[j]);
	printf("\n");
	sumiobs += intens[i].i_iobs;
      }
      printf("Sumiobs= %.1f, %d reflections read.\n\n", sumiobs, nintens);
      fclose(lebail);
      /* end of reading fullprof lebail intensitites and calculating overlaps */
    }
     

    if( strncmp (bufr, "lebailg", 7) == 0){
      init_pluck(bufr+8);
      
      if( (lebail = fopen( pluck(), "r") ) == NULL)
	errout("Can't open GSAS intens file");
      if( (nintens = atoi( pluck() ) ) <= 0) errout("Can't understand nintens");
      if( nintens > NINTENS ) errout("RECOMPILE -- nintens > NINTENS");

      /* READ THE INTEGRATED INTENSITIES FROM THE LE BAIL FIT (GSAS) */

      printf("DATA READ FROM THE GSAS FIT: %s\n", bufr+7);
      printf("------------------------------ \n");      
      printf("  h   k   l     mult.    2theta  sigma      II   Overlap coefficients (Fi,i Fi,i+1 Fi,i+2 Fi,i+3 Fi,i+4)\n"); 
      printf("-------------------------------------------------------------------------------------------------------\n");
      
      
      for(i=0; i<nintens; i++){
	if (fgets(bufr, 200, lebail) == NULL) break;
	sscanf(bufr, "%d %d %d %d %*f %f %f %f", &intens[i].i_hkl[0], &intens[i].i_hkl[1], &intens[i].i_hkl[2], &intens[i].i_m, &intens[i].i_tth, &intens[i].i_wid, &intens[i].i_iobs);
	/* GSAS gives numerical FWHM, which is increased at small angles by asymmetry. */
	intens[i].i_wid /= 2.35; /* now is sigma, assuming it was FWHM */
	t1 = intens[i].i_tth / RADIAN;
	t2 = intens[i].i_m / (sin(t1) * sin(t1/2) * 100000.);
	intens[i].i_iobs *= t2;

	if(i < NINTENS - NOVLP) for(j=1; j<NOVLP; j++) intens[i].i_cij[j] = 0;
	intens[i].i_cij[0] = 1. / (3.5449077 * intens[i].i_wid);
	for(j=1; j<NOVLP; j++){
	  if(i-j < 0) continue;
	  t1 = intens[i-j].i_wid * intens[i-j].i_wid + intens[i].i_wid * intens[i].i_wid;
	  t2 = intens[i-j].i_tth - intens[i].i_tth;
	  t2 *= t2 / (2. * t1);
	  if ( t2 > 10 ) intens[i-j].i_cij[j] = 0;
	  else intens[i-j].i_cij[j] = 1. / (2.506628 * sqrt(t1) * exp(t2) ); 
	}    
      }
      for(i=0; i<nintens; i++){
	printf("%3d %3d %3d %8d\t %3.3f\t %3.3f\t %5.1f ", intens[i].i_hkl[0], intens[i].i_hkl[1], intens[i].i_hkl[2], intens[i].i_m, intens[i].i_tth, intens[i].i_wid, intens[i].i_iobs);
	for(j=0; j<NOVLP; j++) printf("%8.4f ", intens[i].i_cij[j]);
	printf("\n");
	sumiobs += intens[i].i_iobs;
      }
      printf("Sumiobs= %.1f, %d reflections read.\n\n", sumiobs, nintens);
      fclose(lebail);
      /* end of reading gsas intensitites and calculating overlaps */
    }


    /* it reads the lattice parameters and calculates cellconv[3][3]    */
    if( strncmp (bufr, "lattice", 7) == 0){ 
     
      sscanf(bufr+8, "%f %f %f %f %f %f", &cella, &cellb, &cellc, &alpha, &beta, &gama);   

    printf("CELL PARAMETERS: \n");
    printf("-----------------\n");	
      printf("a, b, c, alpha, beta, gamma (Angstrom and deg.) = %.4f %.4f %.4f %.3f %.3f %.3f \n", cella, cellb, cellc, alpha, beta, gama);
      printf("\n");
      
      lattconv();
      printf("Calculated cellconv[3][3] matrix \n");       /* prints the calculated cellconv matrix */
      for(i=0; i<3; i++){  
	for (j=0; j<3; j++)
	  printf(" %f", cellconv[i][j] ); 
	printf(" \n");}
      printf("\n");
      continue;
    }
    
    /* it reads the wavelength  */
    if( strncmp (bufr, "lambda", 6) == 0){ 
      lambda = atof(bufr+6);

    printf("WAVELENGTH: \n");
    printf("-----------\n");	

      printf("Lambda (Angstrom) = %f \n\n", lambda);
      continue;
    }

    /* it reads the space group symbol (only for output files) */
    if( strncmp (bufr, "spacegroup", 10) == 0 ){
      strncpy(space_group_symbol, bufr+11, 100);
    printf("SPACE GROUP SYMMETRY: \n");
    printf("--------------------\n");
      printf("Space group symbol = %s, (string only used in writing output files) \n\n", space_group_symbol);
      continue;
    }

    /* read the symmetry elements for diffraction calculation */
    if( strncmp (bufr, "symm", 4) == 0){
      nsym = atoi (bufr + 4);
      if( nsym > 0) centrosym = 0;
      else{
	centrosym = 1;
	nsym *= -1;
      }

      printf("SIGNS AND TRANSLATIONS OF THE SPACE GROUP SYMMETRY OPERATIONS: \n");
      printf("-------------------------------------------------------------- \n");
      printf("%d symmetry operations, centrosym=%d, ", nsym, centrosym); 

      if (centrosym == 0) 
      printf("non centrosymmetric space group.\n");
      else
      printf("centrosymmetric space group.\n");

      if(nsym > NSYM) errout("RECOMPILE -- nsym > NSYM");

      for(i=0; i<nsym; i++){
	fgets(bufr, MAXBUF, stdin);
	sp = &symelement[i];
	sscanf(bufr, "%f %f %f %f %f %f %f %f %f %f %f %f", &sp->s_m[0][0], &sp->s_m[0][1], &sp->s_m[0][2],
	       &sp->s_m[1][0], &sp->s_m[1][1], &sp->s_m[1][2], &sp->s_m[2][0], &sp->s_m[2][1], &sp->s_m[2][2],
	       &sp->s_t[0], &sp->s_t[1], &sp->s_t[2]);

	printf("%6.2f %6.2f %6.2f %8.2f %6.2f %6.2f %8.2f %6.2f %6.2f %8.2f %6.2f %6.2f\n",
		sp->s_m[0][0], sp->s_m[0][1], sp->s_m[0][2],
		sp->s_m[1][0], sp->s_m[1][1], sp->s_m[1][2], sp->s_m[2][0], sp->s_m[2][1], sp->s_m[2][2],
		sp->s_t[0], sp->s_t[1], sp->s_t[2]);
      }
      printf("\n");
      continue;
    }


     /* read the symmetry elements for diffraction calculation in the non matrix form*/
    if( strncmp (bufr, "symt", 4) == 0)
      {
	nsym = atoi (bufr + 4);
	if( nsym > 0) centrosym = 0;
	else
	  {
	    centrosym = 1;
	    nsym *= -1;
	  }
	
	printf("SIGNS AND TRANSLATIONS OF THE SPACE GROUP SYMMETRY OPERATIONS: \n");
	printf("-------------------------------------------------------------- \n"); 
	printf("%d symmetry operations, centrosym=%d, ", nsym, centrosym);
	
	if (centrosym == 0) 
	  printf("non centrosymmetric space group.\n");
	else
	  printf("centrosymmetric space group.\n");
	
	if(nsym > NSYM) errout("RECOMPILE -- nsym > NSYM");
	
	for(i=0; i<nsym; i++)
	  {
	    sp = &symelement[i];
	    fgets(bufr, MAXBUF, stdin);
	    sp = &symelement[i];
	    for(j=0; j<MAXBUF; j++){
	      if(bufr[j] == '\0') break;
	      if(bufr[j] == ',') errout("I don't want commas in the symmetry specifiers!");
	    }

	    sscanf(bufr, "%s %s %s",word1,word2,word3);
	    strcpy(symtext[i].X,word1);
	    strcpy(symtext[i].Y,word2);
	    strcpy(symtext[i].Z,word3);
	    rot_parse(word1, &rot1);
	    rot_parse(word2, &rot2);
	    rot_parse(word3, &rot3);

	    sp->s_m[0][0] = record.s_m[0][0] = rot1.x;
	    sp->s_m[0][1] = record.s_m[0][1] = rot1.y;
	    sp->s_m[0][2] = record.s_m[0][2] = rot1.z;
	    sp->s_m[1][0] = record.s_m[1][0] = rot2.x;
	    sp->s_m[1][1] = record.s_m[1][1] = rot2.y;
	    sp->s_m[1][2] = record.s_m[1][2] = rot2.z;
	    sp->s_m[2][0] = record.s_m[2][0] = rot3.x;
	    sp->s_m[2][1] = record.s_m[2][1] = rot3.y;
	    sp->s_m[2][2] = record.s_m[2][2] = rot3.z;
	    sp->s_t[0]    = record.s_t[0] = trans_parse(word1);
	    sp->s_t[1]    = record.s_t[1] = trans_parse(word2);
	    sp->s_t[2]    = record.s_t[2] = trans_parse(word3);

	    printf("%s  %s  %s\n",symtext[i].X,symtext[i].Y, symtext[i].Z);
  
	    printf("%6.2f %6.2f %6.2f %8.2f %6.2f %6.2f %8.2f %6.2f %6.2f %8.2f %6.2f %6.2f\n",
		   record.s_m[0][0], record.s_m[0][1], record.s_m[0][2],
		   record.s_m[1][0], record.s_m[1][1], record.s_m[1][2],
		   record.s_m[2][0], record.s_m[2][1], record.s_m[2][2],
		   record.s_t[0], record.s_t[1], record.s_t[2]);
	    if (centrosym == 1)
	      {
		invert_string(word1,invword1);
		invert_string(word2,invword2);
		invert_string(word3,invword3);
		strcpy(syminv[i].X,invword1);
		strcpy(syminv[i].Y,invword2);
		strcpy(syminv[i].Z,invword3);
		printf("%s  %s  %s\n",syminv[i].X,syminv[i].Y,syminv[i].Z); 
	      }
	  }
	printf("\n");
	continue;
      }
    
    // jumppar and algorithm number 
    
    if ( strncmp (bufr, "algorithm", 9) == 0){
      sscanf(bufr+10, "%d %d", &algor, &jumppar);
      printf("Set algorithm number=%d, jumppar=%d\n", algor, jumppar);
      printf("\n");
      continue;
    }

    // number of adjustable parameters, and their ranges

    if ( strncmp (bufr, "params", 6) == 0 ){
      sscanf(bufr+7, "%d", &npar);
      printf("There are %d adjustable parameters.\n", npar);
      if(npar > NPAR) errout("RECOMPILE -- npar > NPAR");
      for(i=0; i<npar; i++){
	fgets(bufr, MAXBUF, stdin);
	if ( sscanf(bufr, "%f %f", &d_range.r_fmin[i], &d_range.r_fmax[i]) != 2)
	  errout("Trouble reading d_range.");
      }
      printf("\n");
      continue;
    }

    // indicates the beginning of each molecule 
    if( strncmp (bufr, "molecule", 8) == 0 ){
      sscanf(bufr+9, "%d %d", &nmol, &natinmol);

      printf("DESCRIPTION OF THE ASYMMETRIC UNIT:\n");
      printf("---------------------------------- \n");

      printf("Molecule number %d has %d atoms.\n", nmol, natinmol);
      if(natinmol > NATINMOL) errout("RECOMPILE -- natinmol > NATINMOL");
      molecule[nmol].m_natoms = natinmol;

      printf("Reading atomic number, atomic symbol, cartesian coordinates and occupancy factor for the molecule number %d with %d atoms.\n\n", nmol, natinmol);    
      printf("atnumber attype atsymbol   X        Y        Z     frac\n");
      printf("---------------------------------------------------------\n\n");

      for(j=0; j<natinmol; j++){      // now load each atom 
	ap = &(molecule[nmol].m_atom[j]);
	if (fgets(bufr, MAXBUF, stdin) == NULL) break;
	sscanf(bufr, "%d %s %f %f %f %f", &ap->a_atnum, ap->a_symbol, &ap->a_xyz[0], &ap->a_xyz[1], 
	       &ap->a_xyz[2], &ap->a_frac);	
	if (ap->a_atnum > MAXATNUM) errout(" molecule contains atomic number too high.");
	if(atype[ap->a_atnum] != -1 ) ap->a_atype = atype[ap->a_atnum];
	else{  // new atom number, assign type and stuff reflection list
	  if(nextatomtype > NATOMTYPE) errout("RECOMPILE, NATOMTYPE too small");
	  printf("assigning attype %d to atomic number %d\n", nextatomtype, ap->a_atnum);
	  if( ac[ap->a_atnum][0]  == 0) errout("WARNING -- atomic number is not listed!\n");
	  atype[ap->a_atnum] = nextatomtype;
	  ap->a_atype = nextatomtype;
	  atnum[nextatomtype] = ap->a_atnum;
	  for(i=0; i<nintens; i++)
	    intens[i].i_fflp[nextatomtype] = fflp(ap->a_atnum, intens[i].i_tth);
	  nextatomtype++;
	}
	printf("%3d %5d %4d %10s %8.4f %8.4f %8.4f %8.4f\n", j+1, ap->a_atnum, ap->a_atype, ap->a_symbol, 
		ap->a_xyz[0], ap->a_xyz[1], ap->a_xyz[2], ap->a_frac);
      }
      printf("\n");
      continue;
    }


    /* IT PRODUCES RANDOM STARTING PARAMETERS */
    if( strncmp (bufr, "random", 6) == 0) {

		
     printf("ASIGNATION OF RANDOM PARAMETERS:\n"); 
     printf("------------------------------- \n");

     for(i=0; i< npar; i++){
	 d0.d_f[i] = d_range.r_fmin[i] + (d_range.r_fmax[i] - d_range.r_fmin[i]) * (rand() / BIG);
      }
      putatoms(&d0);
      d0.d_r = rfactor();
	  
      printf("Random starting params give S=%.3f\n", d0.d_r);
      printpars(&d0);
      printf("\n");
      continue;
    }


    /* It reads in the lines specifying the structure - to be used in putatoms() */
    if( strncmp(bufr, "structure", 9) == 0){
      printf("STRUCTURE definition: molecule number, internal degrees of freedom, Eulerian angles, molecular positions.\n");
      nats = atoi(bufr + 9);
      for(i=0; i<MAXPUTCMD; i++){
	fgets(bufr, MAXBUF, stdin);
	printf("%s\n", bufr);
	if( strncmp(bufr, "endstructure", 12) == 0 )
	  break;
	for(cp=bufr; *cp != '\n' && *cp != '\0'; cp++) ;
	*cp = '\0';   // strip \n from end of line, so commands are terminated by \0.
	strcpy(putcmd[i], bufr);
      }
      printf("The asymmetric unit has %d atoms, defined by %d lines in putcmd.\n\n", nats, i);
      putcmd[i][0] = '\0';
      printf("\n");
      continue;
    }

    // manual putatoms for testing purpose
    if(strncmp(bufr, "putatoms", 8) == 0){
      printf("manually putting atoms into array\n");
      putatoms(&d0);
      for(i=0; i<nats; i++)  /* note that atoms are internally numbered from zero */
	printf("%3d %3d %8.4f %8.4f %8.4f\n", i+1, atoms[i].a_atnum, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2]);
      continue;
      printf("\n");
    }

    if(strncmp(bufr, "printpars", 9) == 0) {
      printpars(&d0);
      printf("\n");
      continue;
    }

       
    /* produce a grid of two of the parameters */
    if(strncmp (bufr, "grid", 4) == 0) {
      init_pluck(bufr+5);
      /* read 2 * {parameter no., minimum, step, maximum} */
      cp = pluck();
      gp0n = atoi(cp);
      cp = pluck();
      p0min = atof(cp);
      cp = pluck();
      p0max = atof(cp);
      cp = pluck();
      p0step = atof(cp);
      cp = pluck();
      gp1n = atoi(cp);
      cp = pluck();
      p1min = atof(cp);
      cp = pluck();
      p1max = atof(cp);
      cp = pluck();
      p1step = atof(cp);
      printf("grid: %d from %f to %f in steps of %f, %d from %f to %f in steps of %f\n", gp0n, p0min, p0max, p0step, gp1n, p1min, p1max, p1step);
      for(gp0 = p0min; gp0 <= p0max; gp0 += p0step)
	for(gp1 = p1min; gp1 <= p1max; gp1 += p1step){
	  d0.d_f[gp0n] = gp0;
	  d0.d_f[gp1n] = gp1;
	  putatoms(&d0);
	  d0.d_r = rfactor();
	  printf("%f %f %f\n", gp0, gp1, d0.d_r);
	}
      continue;
    }


    /* MANUALLY enter parameters */
    if(strncmp (bufr, "enter_p", 7) == 0) {
      printf("PARAMETERS ENTERED:\n");
      printf("------------------ \n");
 
     init_pluck(bufr+8); 
      for(i=0; i<npar; i++){
	cp = pluck();
	d0.d_f[i] = atof(cp);
	printf("%f  ", d0.d_f[i]);
      }
      printf("\n");

      putatoms(&d0);
      d0.d_r = rfactor();

      printf("S = %.3f\n", d0.d_r);
      continue;
    }
	
    /* SIMULATED ANNEALING CYCLES  */

    if( strncmp (bufr, "anneal", 6) == 0) {                         
      sscanf(bufr+7, "%f %f %d %f", &init, &tfac, &ncyc, &fint);
      
      printf("SIMULATED ANNEALING: \n");
      printf("------------------- \n");
      printf("\n");
      
      printf("Algorithm %d, jumppar %d \n", algor, jumppar);
      printf("Initial T=%.2f, T factor=%.2f, number of cycles=%d, final T=%.3f\n", init, tfac, ncyc, fint);
      printf("-------------------------------------------------------------------------------\n");
      printf("\n");
      
      dcopy(&d0, &dbest);     //initial random numbers given in "random" copied to dbest
      putatoms(&dbest);
      dbest.d_r = rfactor(); 
      
      
      for(t = init; t >= fint ; t = tfac*t){            /* LOOP IN TEMPERATURE */
	
	time(&time1);
	nacc=0;   /* number of accepted structures */
	sumr = d0.d_r;
	sumrsq = d0.d_r * d0.d_r;
	
	
	if (algor == 1) {
	  
	  for(cyc=0; cyc<ncyc; cyc++){                       /* CYCLES */
	    
	    dcopy(&d0, &dtest);  	/* load trial parameters into dtest */
	    
	    scale = rand()/BIG;
	    for(i=0; i<jumppar; i++) scale *= scale; /* flat for jumppar=0 */
	    
	    /* simple for now - no limits */
	    for(i=0; i<npar; i++){    // previous d0-dtest parameters plus some small changes kept in dtest
	      span = d_range.r_fmax[i] - d_range.r_fmin[i];
	      if( fabs(span - 360) < .1 )
		dtest.d_f[i] += 180. * scale * (rand()/ALMOSTBIG - 1.); 
	      else if( fabs(span - 1) < .001 )
		dtest.d_f[i] += scale * (rand()/ALMOSTBIG -1.); 
	      else{ 
		dtest.d_f[i] +=  span * scale * (rand()/BIG - 0.5); 
		if(dtest.d_f[i] > d_range.r_fmax[i]) dtest.d_f[i] = d_range.r_fmax[i];
		if(dtest.d_f[i] < d_range.r_fmin[i]) dtest.d_f[i] = d_range.r_fmin[i];
	      } 
	    }
	    
	    putatoms(&dtest);
	    dtest.d_r = rfactor();
	    
	    
	    dif = dtest.d_r - d0.d_r; //Metropolis 
	    if(dif < 0){
	      ok = 1;
	      if(dtest.d_r < dbest.d_r) dcopy(&dtest, &dbest);
	    }
	    else{
	      boltz = exp(-dif / t);
	      ok = (boltz > rand()/BIG) ? 1 : 0;
	    }
	    if(ok){
	      nacc += 1;
	      sumr += dtest.d_r;
	      sumrsq += dtest.d_r * dtest.d_r;
	      dcopy(&dtest, &d0);  //d0 keeps best in neigbourhood or accepted params.
	    } 
	    
	    /* printf("d0 ");   
	       printpars(&d0);
	       printf("dbest ");   
	       printpars(&dbest);
	       printf("\n");
	       printf("\n"); */
	    
	  } /* end of loop in cycles */
	}; /* end algorithm 1 */
	
	
	if (algor == 2) {
	  // random change (big) in one parameter at random in new dtest copied from previous d	  

	  for(cyc=0; cyc<ncyc; cyc++){                       /* CYCLES */
	    dcopy(&d0, &dtest); //get the accepted params in dtest
	  
	    i= abs(rand()/BIG *npar);	//selects parameter number to be changed at random 
	  
	    dtest.d_f[i] = d_range.r_fmin[i] + (d_range.r_fmax[i] - d_range.r_fmin[i]) *  rand()/BIG ; //random value for parameter h
	    
	    putatoms(&dtest);
	    dtest.d_r = rfactor(); 
	    
	    dif = dtest.d_r - d0.d_r;    //Metropolis after change in one parameter in dtest
	    if(dif < 0){
	      ok = 1;
	      if(dtest.d_r < dbest.d_r) dcopy(&dtest, &dbest);
	    }
	    else{
	      boltz = exp(-dif / t);
	      ok = (boltz > rand()/BIG) ? 1 : 0;
	    }
	    if(ok){
	      nacc += 1;
	      sumr += dtest.d_r;
	      sumrsq += dtest.d_r * dtest.d_r;
	      dcopy(&dtest, &d0);   //d0 has the accepted params     
	    } 
	    
	    /* printf("d0 ");   
	       printpars(&d0);
	       printf("dbest ");   
	       printpars(&dbest);
	       printf("\n");
	       printf("\n"); */
	    
	  } /* end of loop in cycles */
	} /* end algorithm 2 */
	
	
	
	dcopy(&dbest, &d0);    //copy dbest to d0 for new neighbourhood when decreasing T.
	sumr /= nacc+1;
	sumrsq /= nacc+1;
	time(&time2);
	
	printf("Temperature=%.4f\n", t);
	printf("%d accepted, average S = %.3f, standard dev= %.4f, best S= %.3f, %d seconds\n", nacc, sumr, sqrt(sumrsq - sumr * sumr), d0.d_r, time2-time1);
	
	for(i=0; i<npar; i++){
	  span = d_range.r_fmax[i] - d_range.r_fmin[i];
	  if( fabs(span - 360) < .1 ){
	    while(d0.d_f[i] > 360) d0.d_f[i] -= 360;
	    while(d0.d_f[i] < 0) d0.d_f[i] += 360;
	  }
	  else if( fabs(span - 1) < .001 ){
	    while(d0.d_f[i] > 1.) d0.d_f[i] -= 1.;      
	    while(d0.d_f[i] < 0.) d0.d_f[i] += 1.;
	  }
	  else if ( d0.d_f[i] < d_range.r_fmin[i] || d0.d_f[i] > d_range.r_fmax[i] )
	    errout("How did parameter get out of limits?");
	}
	
	putatoms(&d0);
	d0.d_r = rfactor();
	printf("Best ", t);  //prints the best parameters obtained at this temperature
	printpars(&d0);
	printf("\n");
	
	fflush(stdout);
      } /* end of loop in temperature */
      
      printf("-------------------------------------------------------------------------------\n");
      printf("This run Best ");
      printpars(&d0);
      printf("-------------------------------------------------------------------------------\n");
      
      printf("\n");
      
      
      continue;
    }
    
    
    // CALCULATE THE DIFFRACION PATTERN FOR CURRENT DESCRIPTION 
    if(strncmp(bufr, "pattern", 7) == 0){
 
      printf("\n");
      printf("DIFFRACTION INTENSITIES FOR THE CURRENT STRUCTURAL DESCRIPTION: \n");
      printf("--------------------------------------------------------------  \n");

      printpars(&d0);  
      printf("\n");     

      printf("h  k  l   2theta  Iobs. Icalc.  Icalc-Iobs  Overlap coefficients (Fi,i Fi,i+1 Fi,i+2 Fi,i+3 Fi,i+4) \n");
      printf("--------------------------------------------------------------------------------------------------- \n");

      for(i=0; i<nintens; i++){
       
       intens[i].i_dif = intens[i].i_icalc - intens[i].i_iobs;

	printf("%2d %2d %2d %7.3f | %6.1f %6.1f %6.1f |  ", intens[i].i_hkl[0], intens[i].i_hkl[1],
		intens[i].i_hkl[2], intens[i].i_tth, intens[i].i_iobs, intens[i].i_icalc, intens[i].i_dif);
	for(j=0; j<NOVLP; j++) printf("%7.3f ", intens[i].i_cij[j]);
	printf("\n");
      }

      printf("Pattern S = %.3f\n\n", d0.d_r);
    }
 
   /* IT WRITES A MOLDRAW INPUT FILE  */
    if( strncmp (bufr, "moldraw", 7) == 0) {
      moldraw_file(bufr+8);
      continue;
    }


    /* IT WRITES A SCHAKAL INPUT FILE  */
    if( strncmp (bufr, "schakal", 7) == 0) {
      schakal_file(bufr+8);
      continue;
    }
   /* IT WRITES A CIF INPUT FILE FOR MERCURY  */

    if( strncmp (bufr, "cif", 3) == 0) {
      cif_file(bufr+4);
      continue;
    }

    /* IT WRITES THE COORDINATES FOR PCR (FULLPROF) FILE */

    if( strncmp (bufr, "fullprofcoords", 14) == 0) {
      fullprof_file(bufr+15);
      continue;
    }
    
    /* IT WRITES THE COORDINATES FOR PCR  complet FULLPROF FILE */
    
    if( strncmp (bufr, "fullprofpcr", 11) == 0) {
      ortep3_file(bufr+12);
      continue;
    }
    
    /* IT WRITES THE COORDINATES FOR GSAS  FILE */

    if( strncmp (bufr, "gsas", 4) == 0) {
      gsas_file(bufr+5);
      continue;
    }
    

  /* IT EXITS THE PROGRAM */
 
    if( strncmp (bufr, "end", 3) == 0){
      printf("END\n");
      exit(0);
    }
    
}    /* end of while loop reading lines of input file */

} /* end of main */
/*--------------------------------------------------------------------------------------------------------------------------------*/
/* FUNCTIONS */
/*----------------------------------------------------------------------------------------------------------------------*/

void lattconv(){            /* it calculates cellconv[3][3]  */

  float a11 =0;
  float a12 =0;
  float a13 =0;
  float a21=0;
  float a22=0;
  float a23=0;
  float a31=0;
  float a32=0;
  float a33=0;
    
  a11= cella;
  a21= cellb * cos(gama/RADIAN);
  a22= cellb * sin(gama/RADIAN);
  a31= cellc * cos(beta/RADIAN);
  a32= (cellb * cellc * cos(alpha/RADIAN) - a21 * a31)/ a22;
  a33= sqrt(cellc*cellc - a31*a31 - a32*a32);
 
  volume = a11*a22*a33;

  cellconv[0][0] =(a22 * a33 - a23 * a32)/volume;
  cellconv[0][1]= -(a21*a33 - a23 * a31)/volume;
  cellconv[0][2]= (a21*a32 - a22*a31)/volume;
  cellconv[1][0]= 0;
  cellconv[1][1]= (a11*a33 - a13*a31)/volume;
  cellconv[1][2] = -(a11*a32 - a12*a31)/volume;
  cellconv[2][0]= 0;
  cellconv[2][1] = 0;  
  cellconv[2][2]= (a11*a22 - a12*a21)/volume;

  printf("Cell volume = %.1f\n\n", volume); 
}  




/* FORM FACTOR COEFFICIENTS TABLE */
float ac[MAXATNUM][9] = {
  {0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 0 */
  {0.493002, 10.5109, 0.322912, 26.1257, 0.140191, 3.14236, 0.040810, 57.7997, 0.003038},  /* H */
  {0.873400, 9.10370, 0.630900, 3.35680, 0.311200, 22.9276, 0.178000, 0.982100, 0.006400}, /* He */
  {0.696800, 4.62370, 0.788800, 1.95570, 0.341400, 0.631600, 0.156300, 10.0953, 0.016700}, /* Li+ */
  {6.26030, 0.002700, 0.884900, 0.831300, 0.799300, 2.27580, 0.164700, 5.11460, -6.1092},  /* Be+2 */
  {2.05450, 23.2185, 1.33260, 1.02100, 1.09790, 60.3498, 0.706800, 0.140300, -0.19320},    /* B */
  {2.31, 20.8439, 1.02, 10.2075, 1.5886, .5687, .865, 51.6512, .2156},       /* C */
  {12.2126, .0057, 3.1322, 9.8933, 2.0125, 28.9975, 1.1663, .5826, -11.529}, /* N */
  {3.0485, 13.2771, 2.2868, 5.7011, 1.5463, .3239, .867, 32.9089, .2508},    /* O */
  {3.53920, 10.2825, 2.64120, 4.29440, 1.51700, 0.261500, 1.02430, 26.1476, 0.277600},  /* F */
  {3.95530, 8.40420, 3.11250, 3.42620, 1.45460, 0.230600, 1.12510, 21.7184, 0.351500},  /* Ne */
  {3.2565, 2.6671, 3.93620, 6.11530, 1.3998, 0.2001, 1.0032, 14.039, 0.404},   /* Na+ */
  {3.49880, 2.16760, 3.83780, 4.75420, 1.32840, 0.185000, 0.849700, 10.1411, 0.485300}, /* Mg+2 */
  {4.174480, 1.93816, 3.38760, 4.14553, 1.20296, 0.228753, 0.528137, 8.28524, 0.706786}, /* Al+3 */
  {6.29150, 2.43860, 3.03530, 32.3337, 1.98910, 0.678500, 1.54100, 81.6937, 1.14070},    /* Si */
  {6.4345, 1.90670, 4.1791, 27.157, 1.78, 0.526, 1.4908, 68.1645, 1.1149}, /* P */
  {6.90530, 1.4679, 5.2034, 22.2151, 1.4379, .2536, 1.5863, 56.172, .8669},  /* S */
  {18.2915, .0066, 7.2084, 1.1717, 6.5337, 19.5424, 2.3386, 60.4486, -16.378}, /* Cl- */
  {7.48450, 0.907200, 6.77230, 14.8407, 0.653900, 43.8983, 1.64420, 33.3929, 1.44450}, /* Ar */
  {7.95780, 12.6331, 7.49170, 0.767400, 6.35900, -0.00200, 1.19150, 31.9128, -4.9978}, /* K+ */
  {15.6348, -0.00740, 7.95180, 0.608900, 8.43720, 10.3116, 0.853700, 25.9905, -14.875}, /* Ca+2 */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Sc */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ti */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* V */
  {9.54034, 5.66078, 7.75090, 0.344261, 3.58274, 13.3075, 0.509107, 32.4224, 0.616898}, /* Cr+2 */
  {10.8061, 5.27960, 7.36200, 0.343500, 3.52680, 14.3430, 0.218400, 41.3235, 1.08740}, /* Mn+2 */
  {11.1764, 4.6147, 7.3863, .3005, 3.3948, 11.6729, .0724, 38.5566, 0.9707}, /* Fe 3+ */
  {11.2296, 4.12310, 7.38830, 0.272600, 4.73930, 10.2443, 0.710800, 25.6466, 0.932400}, /* Co+2 */
  {11.4166, 3.67660, 7.40050, 0.244900, 5.34420, 8.87300, 0.977300, 22.1626, 0.861400}, /* Ni+2 */
  {11.8168, 3.37484, 7.11181, 0.244078, 5.78135, 7.98760, 1.14523, 19.8970, 1.14431},   /* Cu+2 */
  {11.9719, 2.99460, 7.38620, 0.203100, 6.46680, 7.08260, 1.39400, 18.0995, 0.780700},  /* Zn+2 */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ga */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ge */
  {16.6723, 2.63450, 6.07010, 0.264700, 3.43130, 12.9479, 4.27790, 47.7972, 2.53100},  /* As */
  {17.0006, 2.40980, 5.81960, 0.272600, 3.97310, 15.2372, 4.35430, 43.8163, 2.84090},  /* Se */
  {17.1789, 2.17230, 5.23580, 16.5796, 5.63770, 0.260900, 3.98510, 41.4328, 2.95570},  /* Br */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Kr */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Rb */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Sr */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Y */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Zr */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Nb */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Mo */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Tc */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ru */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Rh */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Pd */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ag */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Cd */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* In */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Sn */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Sb */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Te */
  {20.1472, 4.34700, 18.9949, 0.381400, 7.51380, 27.7660, 2.27350, 66.8776, 4.07120},  /* I */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Xe */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Cs */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ba */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* La*/
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ce */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Pr */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Nd */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Pm */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Sm */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Eu */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Gd*/
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Tb */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Dy */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ho */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Er */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Tm */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Yb */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Lu*/
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Hf */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /* Ta */
  {0, 0, 0, 0, 0, 0, 0, 0, 0},  /*  W */
  {28.7621, 1.67191, 15.7189, 9.092270,14.55640,  0.3505, 5.44174, 52.0861,10.47200},  /* Re*/

};  

/*----------------------------------------------------------------------------------------------------------------------------*/

 float fflp(int atom, float tth){                   /* it calculates the atom form factor*sqrt(Lorentz polarization factor) */
  int i;
float ff, q, lp;

q = sin(tth/(2.0 * RADIAN));
lp = (sin(tth/RADIAN)*q);
q /= lambda;

ff = ac[atom][8];
for(i=0; i<4; i++) ff += ac[atom][2*i] / exp(ac[atom][2*i+1] *q *q);
return(ff/sqrt(lp));
 }

/*----------------------------------------------------------------------------------------------------------------------------*/

void moldraw_file(char *filename){                 /* it writes a moldraw input file */ 
  
  int i;
  FILE *mol;
  printf("VISUALIZATION FILES: An input file for the program  MOLDRAW, called %s has been printed.\n", filename);
  printf("-------------------\n");
  printf("\n");

  mol = fopen(filename, "w");
  fprintf(mol, "TITLE  \n");
  fprintf(mol, "%s \n", title);
  fprintf(mol, "CELL \n");
  fprintf(mol, "%f %f %f %f %f %f \n", cella, cellb, cellc, alpha, beta, gama);
  fprintf(mol, "COORD \n");
  for(i=0; i< nats ; i++)   /* nats real number of atoms in molecule */  
    fprintf(mol, "%d %f %f %f \n", atoms[i].a_atnum, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2]);
  fprintf(mol, "0   0   0   0 \n");
  fprintf(mol, "GROUP \n");
  fprintf(mol, "%s \n", space_group_symbol);
  fclose(mol);
}

/*----------------------------------------------------------------------------------------------------------------------------*/

void schakal_file(char *filename){                  /* it writes a schakal input file  */
  FILE *dat;
  int i;
  printf("VISUALIZATION FILES: An input file for the program SCHAKAL, called %s has been printed.\n", filename);
  printf("-------------------\n");
  printf("\n");

  dat = fopen(filename, "w");
  fprintf(dat, "TITL   ");
  fprintf(dat, "%s \n", title);
  fprintf(dat, "CELL   ");
  fprintf(dat, "A=%f B=%f C=%f AL=%f BE=%f GA=%f \n", cella, cellb, cellc, alpha, beta, gama);
  for(i=0; i<nats ; i++) 
    fprintf(dat, "ATOM  %s %f %f %f \n", atoms[i].a_symbol, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2]); 
  fprintf(dat, "SPGR %s", space_group_symbol);
  fclose(dat);
} 
/*----------------------------------------------------------------------------------------------------------------------------*/

void cif_file(char *filename){                  /* it writes a bare min cif file for mercury  */
 FILE *cif;
  int i;
  printf("VISUALIZATION FILES: An input file for the program MERCURY, called %s has been printed.\n", filename);
  printf("-------------------\n");
  printf("\n");

  cif = fopen(filename, "w");
  fprintf(cif,"#===================================================================================\n\n");
  fprintf(cif,"data_%s\n\n",title);
  fprintf(cif,"#===================================================================================\n\n");
  fprintf(cif,"# 1. POWDER SPECIMEN AND CRYSTAL DATA\n\n");
  fprintf(cif,"_symmetry_space_group_name_H-M '%s'\n\n",space_group_symbol);
  fprintf(cif,"loop_ \n");
  fprintf(cif,"\t _symmetry_equiv_pos_site_id\n");
  fprintf(cif,"\t _symmetry_equiv_pos_as_xyz\n");
  for(i=0;i < nsym;i++)
    {
      fprintf(cif,"%d %s,%s,%s\n",i+1,symtext[i].X,symtext[i].Y,symtext[i].Z);
    }

  if(centrosym == 1)
    {
      for(i=0;i < nsym;i++)
	{
	  fprintf(cif,"%d %s,%s,%s\n",i+nsym+1,syminv[i].X,syminv[i].Y,syminv[i].Z);
	}
    }

  fprintf(cif,"\n");
  fprintf(cif,"_cell_length_a     %f\n",cella);
  fprintf(cif,"_cell_length_b     %f\n",cellb);
  fprintf(cif,"_cell_length_c     %f\n",cellc);
  fprintf(cif,"_cell_angle_alpha %f\n",alpha);
  fprintf(cif,"_cell_angle_beta  %f\n",beta);
  fprintf(cif,"_cell_angle_gamma %f\n",gama);
  fprintf(cif,"_cell_volume       %f\n",volume);
  fprintf(cif,"#===================================================================================\n\n");
  fprintf(cif,"# 2. ATOMIC COORDINATES AND DISPLACEMENT PARAMETERS\n\n");
  fprintf(cif,"loop_\n");
  fprintf(cif,"  _atom_site_label\n");
  fprintf(cif,"  _atom_site_fract_x\n");
  fprintf(cif,"  _atom_site_fract_y\n");
  fprintf(cif,"  _atom_site_fract_z\n");
  fprintf(cif,"  _atom_site_U_iso_or_equiv\n");
  fprintf(cif,"  _atom_site_thermal_displace_type\n");
  fprintf(cif,"  _atom_site_occupancy\n");
  fprintf(cif,"  _atom_site_type_symbol\n");
  for(i=0; i<nats ; i++) 
    fprintf(cif, "%s%d %f %f %f 0.025 Uiso %f %s  \n", atoms[i].a_symbol,i+1, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2],atoms[i].a_frac, atoms[i].a_symbol); 
  fclose(cif);
}  
/*----------------------------------------------------------------------------------------------------------------------------*/

void fullprof_file(char *filename){                 /* it writes the coordinates to insert in a fullprof (pcr) file */ 
  
  int i;
  FILE *ful;
  printf("The atomic coordinates in FULLPROF format have been printed in file called %s.\n", filename);
  printf("\n");

  ful = fopen(filename, "w");
  fprintf(ful, "This file contains the atomic coordinates in FULLPROF format. Please insert them in a *.pcr file using a text editor.\n");
  fprintf(ful, "Experiment: ");
  fprintf(ful, "%s \n", title);
  for(i=0; i< nats ; i++) {  /* nats real number of atoms in molecule */  
  fprintf(ful, "%s%d  %s   %f %f %f 0.0 %f 0   0   0 \n", atoms[i].a_symbol, i, atoms[i].a_symbol, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2], atoms[i].a_frac); 
  fprintf(ful, "         0.000000 0.000000 0.000000 0.0 0.000000 0   0   0 \n");}
  fclose(ful);
}

/*----------------------------------------------------------------------------------------------------------------------------*/
void ortep3_file(char *filename){                 /* it writes a ortep3 for windows complete input file for fullprof */ 
  
  int i;
  FILE *ortep3;
  printf("An input file for the program FULLPROF, called %s has been printed. Some parameters are appropiate for data taken at the X3B1 beamline, BNL.\n", filename);
  printf("The crystal structure can be visualized with the program ORTEP-3 for Windows. \n");
  printf("\n");

  ortep3 = fopen(filename, "w");
  fprintf(ortep3, "COMMC: ");
  fprintf(ortep3, "%s \n", title);
  fprintf(ortep3, "! Files => DAT-file: **,  PCR-file: ** \n");
  fprintf(ortep3, "!Job Npr Nph Nba Nex Nsc Nor Dum Iwg Ilo Ias Res Ste Nre Cry Uni Cor  \n");
  fprintf(ortep3, "   0  12   1   0   0   0   1   1   0   0   1   0   0   0   0   0   0  \n");
  fprintf(ortep3, "!                                                                     \n");
  fprintf(ortep3, "!Ipr Ppl Ioc Mat Pcr Ls1 Ls2 Ls3 Syo Prf Ins Rpa Sym Hkl Fou Sho Ana  \n");
  fprintf(ortep3, "   0   0   1   1   1   0   0   0   1   1   4   1   1   1   0   0   0  \n");
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "! lambda1 Lambda2    Ratio   Bkpos    Wdt    Cthm     muR   AsyLim   Rpolarz \n");
  fprintf(ortep3, "%f %f 1.0000 15.1000 20.0000  0.7899  0.0000   34.00    0.0000 \n", lambda, lambda);
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "!NCY Eps R_at R_an R_pr R_gl     Thmin       Step       Thmax    PSD    Sent0 \n");
  fprintf(ortep3, "  5 0.30 0.50 0.50 0.50 0.50      2.0000     0.0020    40.0000  0.000  0.000  \n");
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "       7    !Number of refined parameters \n");
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "!  Zero    Code    Sycos    Code   Sysin    Code  Lambda     Code MORE  \n");
  fprintf(ortep3, "  0.0000    0.00  0.0000    0.00  0.0000    0.00 0.000000    0.00   0  \n");
  fprintf(ortep3, "0.1    0.1    0.1   0.01    0.01    0.01 \n"); 
  fprintf(ortep3, "21.0  31.0   41.0   51.0    61.0    71.0  \n");
  fprintf(ortep3, "!------------------------------------------------------------------------------- \n");
  fprintf(ortep3, "!  Data for PHASE number:   1  ==> Current R_Bragg:    none  \n");
  fprintf(ortep3, "!------------------------------------------------------------------------------- \n");
  fprintf(ortep3, "%s \n", title);
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "!Nat Dis Mom Pr1 Pr2 Pr3 Jbt Irf Isy Str Furth     ATZ   Nvk Npr More \n");
  fprintf(ortep3, "  %d 0   0 0.0 0.0 0.0   0   0   0   0   0        0.00   0  12   0  \n", nats );
  fprintf(ortep3, "!\n"); 
  fprintf(ortep3, "%s                  <--Space group symbol \n", space_group_symbol);
  fprintf(ortep3, "!Atom Typ       X        Y        Z     Biso      Occ     In Fin N_t  /Codes \n");

  for(i=0; i< nats ; i++) {  /* nats real number of atoms in molecule */  
    fprintf(ortep3, "%s%d  %s   %f %f %f 0.0 %f 0   0   0 \n", atoms[i].a_symbol, i+1, atoms[i].a_symbol, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2], atoms[i].a_frac); 
    fprintf(ortep3, "         0.000000 0.000000 0.000000 0.0 0.000000 0   0   0 \n");
  }

  fprintf(ortep3, "!  Scale       Shape1      Bov     Str1     Str2     Str3    Strain-Model \n");
  fprintf(ortep3, "0.10000E-03    0.5000   0.0000   0.0000   0.0000   0.0000       0  \n");
  fprintf(ortep3, "     11.00000     0.00     0.00     0.00     0.00     0.00  \n");
  fprintf(ortep3, "!      U        V         W          X         Y       GauSiz   LorSiz Size-Model  \n");
  fprintf(ortep3, "   0.00000   0.00000   0.00025   0.00000   0.00000   0.00000   0.00000   0  \n");
  fprintf(ortep3, "       0.00      0.00      0.00      0.00      0.00      0.00      0.00  \n");
  fprintf(ortep3, "!     a          b         c        alpha      beta       gamma \n");
  fprintf(ortep3, "%f %f %f %f %f %f \n", cella, cellb, cellc, alpha, beta, gama);
  fprintf(ortep3, "    0.00000    0.00000    0.00000    0.00000    0.00000    0.00000  \n");
  fprintf(ortep3, "!  Pref1  Pref2   Asy1    Asy2    Asy3     Asy4  \n");
  fprintf(ortep3, " 1.00000 0.00000 0.00000 0.00000 0.00000 0.00000  \n");
  fprintf(ortep3, "    0.00    0.00    0.00    0.00    0.00    0.00 \n");
  fprintf(ortep3, "!Additional Shape parameters \n");
  fprintf(ortep3, "   0.01200    0.00   0.01200    0.00            SHAPE: SHP1 CSHP1 & SHP2 CSHP2  \n");
  fclose(ortep3);
}

/*----------------------------------------------------------------------------------------------------------------------------*/

void gsas_file(char *filename){                 /* it writes the coordinates to insert in a gsas file */ 
  
  int i;
  FILE *gda;
  printf("The coordinates have been printed in a format appropiate for the program GSAS, in file called %s.\n", filename);
  printf("Please insert them in the respective GSAS menu, typing: @r, return, filename, return.\n");
  printf("\n");

  gda = fopen(filename, "w");
  for(i=0; i< nats ; i++)   /* nats real number of atoms in molecule */  
    fprintf(gda, "i %d %s %f %f %f %f / i / \n",i+1, atoms[i].a_symbol, atoms[i].a_xyz[0], atoms[i].a_xyz[1], atoms[i].a_xyz[2], atoms[i].a_frac);
  fclose(gda);
}

/*----------------------------------------------------------------------------------------------------------------------------*/

void dcopy(struct descr *d1, struct descr *d2){
  int i;
  for(i=0; i<npar; i++) d2->d_f[i] = d1->d_f[i];
  d2->d_r = d1->d_r;
}


/*----------------------------------------------------------------------------------------------------------------------------*/
void printpars(struct descr *dp){
  int i;
  printf("Parameters: ");
  for(i=0; i<npar; i++) printf("%.4f ", dp->d_f[i]);
  printf("\n");
  printf("S=%.3f \n", dp->d_r);
}

/*----------------------------------------------------------------------------------------------------------------------------*/

void errout(char *string){
  printf("%s\nI die.", string);
  printf("Program terminates with error message\n%s\n", string);
  exit(0);
}
/*----------------------------------------------------------------------------------------------------------------------------*/
float trans_parse (char string[])
{
   int i=0;
   float frac = 0;
   float n,d;
   while(string[i] != '\0')
   {
   	if(string[i] == '/')
   	{
      n = (string[i-1]-'0');
      d = (string[i+1]-'0');
      if (n < 0 || n > 9 || d < 0 || d > 9)
      printf("Error in parsing fraction");
      if (string[i-2] == '-')
      	frac = -n/d;
      else
         frac = n/d;
       }
   i++;
    }
    return frac;
}
/*----------------------------------------------------------------------------------------------------------------------------*/

void rot_parse (char string[], struct pt_rec *rot)
{

int j=0;
 (*rot).x = 0;
 (*rot).y = 0;
 (*rot).z = 0;

while(string[j] != '\0')
{
	if(string[j] == 'x')
    { if (string[j-1] == '-')
   		(*rot).x = -1;
   	else (*rot).x = 1;
      }
   if(string[j] == 'y')
    { if (string[j-1] == '-')
   		(*rot).y = -1;
   	else (*rot).y = 1;
      }
   if(string[j] == 'z')
    { if (string[j-1] == '-')
   		(*rot).z = -1;
   	else (*rot).z = 1;
      }
     j++;
}


     return;

}

/*----------------------------------------------------------------------------------------------------------------------------*/
void invert_string(char string[], char inv_str[])
{

  int j=0;
  int i=j+1;
  int N=0,D=0;
  inv_str[0]=' ';

while(string[j] != '\0')
{
    if(string[j] == 'x')
    { if (string[j-1] == '-')
      {
	inv_str[i] = 'x';
	inv_str[i-1] = '+';
      }
    else
      {
      inv_str[i] = 'x';
      inv_str[i-1] = '-';
      }
    }
    if(string[j] == 'y')
    { if (string[j-1] == '-')
      {
	inv_str[i] = 'y';
	inv_str[i-1] = '+';
      }
    else
      {
      inv_str[i] = 'y';
      inv_str[i-1] = '-';
      }
    }
    if(string[j] == 'z')
    { if (string[j-1] == '-')
      {
	inv_str[i] = 'z';
	inv_str[i-1] = '+';
      }
    else
      {
      inv_str[i] = 'z';
      inv_str[i-1] = '-';
      }
    } 
    if(string[j] == '/')
    { 
       N = (string[j+1]-'0')-(string[j-1]-'0');
       D = (string[j+1]-'0');
	inv_str[i] = '/';
	inv_str[i-2]=string[j-2];
	inv_str[i-1] = (N+'0');
	inv_str[i+1] = (D+'0');
     }
   
    if(string[j] == ',')
      inv_str[i]=',';
     j++;
     i++;
}

inv_str[i] = '\0';
     return;

}







